# -----------------------------------------------------------------------------
# Command to color volumes using masks.
#
#   Syntax: mask <volumeSpec> <maskSpec>
#
def mask_command(cmdname, args):

    from Midas.midas_text import doExtensionFunc
    doExtensionFunc(mask, args,
                    specInfo = [('volumeSpec','volumes','models'),
                                ('maskSpec', 'mask', 'models'),])

# -----------------------------------------------------------------------------
#
def mask(volumes, mask):

    from Midas import MidasError

    from VolumeFilter.vopcommand import filter_volumes
    volumes = filter_volumes(volumes)
    mask = filter_volumes(mask)
    if len(mask) > 1:
       raise MidasError, 'More than one mask specified'
    mask = mask[0]

    for v in volumes:
    	mask_coloring(v, mask)    

# -----------------------------------------------------------------------------
#
def mask_coloring(v, m):

    from numpy import uint8, uint16
    t = m.data.value_type
    if t == uint8: s = 256
    elif t == uint16: s = 65536
    else:
        raise TypeError, "Mask must have type uint8 or uint16"
    cmap = colormap(s,4)

    def cm(colors, slice = None, v=v, m=m, cmap=cmap):
        mask = m.region_matrix(v.subregion())
        if slice:
            mask = mask[slice]
        # Numpy is 20x slower than _volume C++ routine.
        # colors[...] *= cmap[mask]
        import _volume
        _volume.indices_to_colors(mask, cmap, colors, modulate = True)

    v.mask_colors = cm

    # Update display.
    if v.shown() and v.representation == 'solid':
	v.close_solid()
	v.show()
    
# -----------------------------------------------------------------------------
# Returns uint8 array of random colors except first element is white.
#
def colormap(n, nc):

    colors = ((1,0,0,1), (0,1,0,1), (0,0,1,1), (1,1,0,1),
              (1,0,1,1), (0,1,1,1), (1,0.5,0,1), (1,0,0.5,1))
    m = len(colors)
    from numpy import empty, single as floatc, uint8
    cmap = empty((n,nc), floatc)
    from random import random
    for c in range(nc-1):
        for k in range(1,n):
            cmap[k,:] = colors[k % m]
    cmap[:,nc-1] = 1      # Opaque
    cmap[0,:] = 1         # First entry is white

    icmap = empty(cmap.shape, uint8)
    import _volume
    _volume.colors_float_to_uint(cmap, icmap)
    
    return icmap
